var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/attrs.ts
import camelCase from "lodash.camelcase";
import { isStyleProp } from "@chakra-ui/styled-system";
import memoize from "lodash.memoize";
var camelCaseCache = {};
var _isStyledProp = memoize((attr) => isStyleProp(attr));
var extractStyleAttrs = /* @__PURE__ */ __name((styleProps) => {
  const styles = {};
  const attrs = {};
  for (const prop in styleProps) {
    let _attr;
    if (camelCaseCache[prop]) {
      _attr = camelCaseCache[prop];
    } else {
      _attr = `${prop.startsWith("_") ? "_" : ""}${camelCase(prop)}`;
      camelCaseCache[prop] = _attr;
    }
    if (_isStyledProp(_attr)) {
      styles[_attr] = styleProps[prop];
    } else if (_isStyledProp(prop)) {
      styles[_attr] = styleProps[prop];
    } else {
      attrs[prop] = styleProps[prop];
    }
  }
  return {
    styles,
    attrs
  };
}, "extractStyleAttrs");

// src/configurable.ts
import { canUseDOM } from "@chakra-ui/utils";
var defaultWindow = canUseDOM() ? window : null;

// src/dom-query.ts
function getSelector(node) {
  const id = node.getAttribute("id");
  if (id)
    return "#" + id;
  let path = "";
  while (node) {
    let name = node.localName;
    const parent = node.parentNode;
    if (!parent) {
      path = name + " > " + path;
      continue;
    }
    if (node.getAttribute("id")) {
      path = "#" + node.getAttribute("id") + " > " + path;
      break;
    }
    const sameTagSiblings = [];
    let children = parent.childNodes;
    children = Array.prototype.slice.call(children);
    children.forEach((child) => {
      if (child.localName == name) {
        sameTagSiblings.push(child);
      }
    });
    if (sameTagSiblings.length > 1) {
      const index = sameTagSiblings.indexOf(node);
      name += ":nth-of-type(" + (index + 1) + ")";
    }
    if (path) {
      path = name + " > " + path;
    } else {
      path = name;
    }
    node = parent;
  }
  return path;
}
__name(getSelector, "getSelector");

// src/dom.ts
import { customRef, onBeforeUpdate, ref, unref } from "vue";

// src/timers.ts
function debounce(func, wait, immediate) {
  let timeout;
  return (...args) => {
    if (immediate && !timeout)
      func(...args);
    clearTimeout(timeout);
    timeout = setTimeout(() => {
      func(...args);
    }, wait);
  };
}
__name(debounce, "debounce");

// src/dom.ts
function useRef() {
  const refEl = ref(null);
  onBeforeUpdate(() => {
    refEl.value = null;
  });
  const _ref = /* @__PURE__ */ __name((el) => {
    var _a;
    refEl.value = (_a = el == null ? void 0 : el.$el) != null ? _a : el;
  }, "_ref");
  return [_ref, refEl];
}
__name(useRef, "useRef");
function unrefElement(elementRef) {
  var _a;
  const node = unref(elementRef);
  return (_a = node == null ? void 0 : node.$el) != null ? _a : node;
}
__name(unrefElement, "unrefElement");
function useDebouncedRef(initialValue, delay = 300, immediate = false) {
  const state = ref(initialValue);
  const debouncedRef = customRef((track, trigger) => ({
    get() {
      track();
      return state.value;
    },
    set: debounce((value) => {
      state.value = value;
      trigger();
    }, delay, immediate)
  }));
  return debouncedRef;
}
__name(useDebouncedRef, "useDebouncedRef");
function contains(containers, element) {
  for (let container of containers) {
    if (container.contains(element))
      return true;
  }
  return false;
}
__name(contains, "contains");

// src/focus.ts
import { getAllFocusable } from "@chakra-ui/utils";
var Focus = /* @__PURE__ */ ((Focus2) => {
  Focus2[Focus2["First"] = 1] = "First";
  Focus2[Focus2["Previous"] = 2] = "Previous";
  Focus2[Focus2["Next"] = 4] = "Next";
  Focus2[Focus2["Last"] = 8] = "Last";
  Focus2[Focus2["WrapAround"] = 16] = "WrapAround";
  Focus2[Focus2["NoScroll"] = 32] = "NoScroll";
  return Focus2;
})(Focus || {});
var FocusResult = /* @__PURE__ */ ((FocusResult2) => {
  FocusResult2[FocusResult2["Error"] = 0] = "Error";
  FocusResult2[FocusResult2["Overflow"] = 1] = "Overflow";
  FocusResult2[FocusResult2["Success"] = 2] = "Success";
  FocusResult2[FocusResult2["Underflow"] = 3] = "Underflow";
  return FocusResult2;
})(FocusResult || {});
function focusElement(element) {
  element == null ? void 0 : element.focus({
    preventScroll: true
  });
}
__name(focusElement, "focusElement");
function focusIn(container, focus) {
  let elements = Array.isArray(container) ? container.slice().sort((a, z) => {
    let position = a.compareDocumentPosition(z);
    if (position & Node.DOCUMENT_POSITION_FOLLOWING)
      return -1;
    if (position & Node.DOCUMENT_POSITION_PRECEDING)
      return 1;
    return 0;
  }) : (() => {
    const focusables = getAllFocusable(container).filter((el) => el !== container);
    return focusables;
  })();
  let active = document.activeElement;
  let direction = (() => {
    if (focus & (1 /* First */ | 4 /* Next */))
      return 1 /* Next */;
    if (focus & (2 /* Previous */ | 8 /* Last */))
      return -1 /* Previous */;
    throw new Error("Missing Focus.First, Focus.Previous, Focus.Next or Focus.Last");
  })();
  let startIndex = (() => {
    if (focus & 1 /* First */)
      return 0;
    if (focus & 2 /* Previous */)
      return Math.max(0, elements.indexOf(active)) - 1;
    if (focus & 4 /* Next */)
      return Math.max(0, elements.indexOf(active)) + 1;
    if (focus & 8 /* Last */)
      return elements.length - 1;
    throw new Error("Missing Focus.First, Focus.Previous, Focus.Next or Focus.Last");
  })();
  let focusOptions = focus & 32 /* NoScroll */ ? {
    preventScroll: true
  } : {};
  let offset = 0;
  let total = elements.length;
  let next = void 0;
  do {
    if (offset >= total || offset + total <= 0)
      return 0 /* Error */;
    let nextIdx = startIndex + offset;
    if (focus & 16 /* WrapAround */) {
      nextIdx = (nextIdx + total) % total;
    } else {
      if (nextIdx < 0)
        return 3 /* Underflow */;
      if (nextIdx >= total)
        return 1 /* Overflow */;
    }
    next = elements[nextIdx];
    next == null ? void 0 : next.focus(focusOptions);
    offset += direction;
  } while (next !== document.activeElement);
  if (!next.hasAttribute("tabindex"))
    next.setAttribute("tabindex", "0");
  return 2 /* Success */;
}
__name(focusIn, "focusIn");

// src/generate-id.ts
function genId(size = 3) {
  let uuid = "";
  const dictionary = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
  for (let i = 0; i < size; i++) {
    uuid += dictionary.charAt(Math.floor(Math.random() * dictionary.length));
  }
  if (process.env.NODE_ENV === "test")
    return "EMPTY_STRING";
  return uuid;
}
__name(genId, "genId");

// src/keys.ts
var Keys = /* @__PURE__ */ ((Keys2) => {
  Keys2["Space"] = " ";
  Keys2["Enter"] = "Enter";
  Keys2["Escape"] = "Escape";
  Keys2["Backspace"] = "Backspace";
  Keys2["ArrowLeft"] = "ArrowLeft";
  Keys2["ArrowUp"] = "ArrowUp";
  Keys2["ArrowRight"] = "ArrowRight";
  Keys2["ArrowDown"] = "ArrowDown";
  Keys2["Home"] = "Home";
  Keys2["End"] = "End";
  Keys2["PageUp"] = "PageUp";
  Keys2["PageDown"] = "PageDown";
  Keys2["Tab"] = "Tab";
  return Keys2;
})(Keys || {});

// src/layout.ts
function orient(options) {
  const {
    orientation,
    vertical,
    horizontal
  } = options;
  if (!orientation)
    return {};
  return orientation === "vertical" ? vertical : horizontal;
}
__name(orient, "orient");

// src/match.ts
function match(value, lookup, ...args) {
  if (value in lookup) {
    let returnValue = lookup[value];
    return typeof returnValue === "function" ? returnValue(...args) : returnValue;
  }
  let error = new Error(`Tried to handle "${value}" but there is no handler defined. Only defined handlers are: ${Object.keys(lookup).map((key) => `"${key}"`).join(", ")}.`);
  if (Error.captureStackTrace)
    Error.captureStackTrace(error, match);
  throw error;
}
__name(match, "match");

// src/props.ts
import { computed } from "vue";
import { filterUndefined } from "@chakra-ui/utils";
var vueThemingProps = {
  colorScheme: String,
  variant: String,
  size: String,
  styleConfig: String
};
var SNA = [Number, String, Array];
var SAO = [String, Array, Object];
var SNAO = [Number, String, Array, Object];
var useThemingProps = /* @__PURE__ */ __name((props) => computed(() => filterUndefined({
  colorScheme: props.colorScheme,
  variant: props.variant,
  size: props.size,
  styleConfig: props.styleConfig
})), "useThemingProps");

// src/scope.ts
import { getCurrentScope, onScopeDispose } from "vue";
function tryOnScopeDispose(fn) {
  if (getCurrentScope()) {
    onScopeDispose(fn);
    return true;
  }
  return false;
}
__name(tryOnScopeDispose, "tryOnScopeDispose");

// src/string.ts
import _camelCase from "lodash.camelcase";
import memoize2 from "lodash.memoize";
var camelCase2 = memoize2((key) => _camelCase(key));

// src/vue-utils.ts
import { isObject } from "@chakra-ui/utils";
import { inject, provide, isVNode } from "vue";
function createContext(options = {}, defaults) {
  const {
    strict = true,
    errorMessage = "useContext: `context` is undefined. Seems you forgot to wrap component within the Provider",
    name
  } = options;
  let contextSymbol = Symbol(`${name}Symbol`);
  function Provider(payload) {
    provide(contextSymbol, payload);
  }
  __name(Provider, "Provider");
  function useContext(fallback = null) {
    const context = inject(contextSymbol, fallback);
    if (!context && strict) {
      throw new Error(errorMessage);
    }
    return context;
  }
  __name(useContext, "useContext");
  return [Provider, useContext, contextSymbol];
}
__name(createContext, "createContext");
function getValidChildren(slots) {
  var _a;
  const slotArray = ((_a = slots == null ? void 0 : slots.default) == null ? void 0 : _a.call(slots)) || [];
  return slotArray.filter((child) => {
    return isVNode(child);
  });
}
__name(getValidChildren, "getValidChildren");
function isObjectComponent(subject) {
  const validComponentTypes = ["function", "object"];
  if (!validComponentTypes.includes(typeof subject))
    return false;
  if (isObject(subject)) {
    if (typeof (subject == null ? void 0 : subject.render) === "function" && isVNode(subject.render()))
      return true;
    else if (typeof (subject == null ? void 0 : subject.setup) === "function")
      return true;
  }
  return false;
}
__name(isObjectComponent, "isObjectComponent");
export {
  Focus,
  FocusResult,
  Keys,
  SAO,
  SNA,
  SNAO,
  camelCase2 as camelCase,
  canUseDOM,
  contains,
  createContext,
  debounce,
  defaultWindow,
  extractStyleAttrs,
  focusElement,
  focusIn,
  genId,
  getSelector,
  getValidChildren,
  isObjectComponent,
  match,
  orient,
  tryOnScopeDispose,
  unrefElement,
  useDebouncedRef,
  useRef,
  useThemingProps,
  vueThemingProps
};
